@@ -8,7 +8,7 @@ import (
 	"net/http"
 
 	"github.com/aws/copilot-cli/e2e/internal/client"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 )
 
@@ -17,7 +17,7 @@ type countAssertionTracker struct {
 	actual   int
 }
 
-var _ = Describe("App Runner", func() {
+var _ = Describe("App Runner", Ordered, func() {
 
 	var (
 		initErr error
@@ -86,39 +86,6 @@ var _ = Describe("App Runner", func() {
 		})
 	})
 
-	Context("run storage init and svc deploy to create an S3 bucket and an aurora DB", func() {
-		var (
-			s3StorageInitErr error
-			auroraInitErr    error
-			deployErr        error
-		)
-
-		BeforeAll(func() {
-			_, s3StorageInitErr = cli.StorageInit(&client.StorageInitRequest{
-				StorageName:  "s3storage",
-				StorageType:  "S3",
-				WorkloadName: feSvcName,
-			})
-			_, auroraInitErr = cli.StorageInit(&client.StorageInitRequest{
-				StorageName:   "front-end-cluster",
-				StorageType:   "Aurora",
-				WorkloadName:  feSvcName,
-				RDSEngine:     "PostgreSQL",
-				InitialDBName: "mydb",
-			})
-			_, deployErr = cli.SvcDeploy(&client.SvcDeployInput{
-				EnvName: envName,
-				Name:    feSvcName,
-			})
-		})
-
-		It("should not return an error", func() {
-			Expect(s3StorageInitErr).NotTo(HaveOccurred())
-			Expect(auroraInitErr).NotTo(HaveOccurred())
-			Expect(deployErr).NotTo(HaveOccurred())
-		})
-	})
-
 	Context("create and deploy a backend service", func() {
 		var (
 			initErr   error
@@ -128,10 +95,10 @@ var _ = Describe("App Runner", func() {
 		BeforeAll(func() {
 			_, initErr = cli.SvcInit(&client.SvcInitRequest{
 				Name:        beSvcName,
-				SvcType:     "Backend Service",
+				SvcType:     "Request-Driven Web Service",
 				Dockerfile:  "./back-end/Dockerfile",
 				SvcPort:     "80",
-				IngressType: "Internet",
+				IngressType: "Environment",
 			})
 			_, deployErr = cli.SvcDeploy(&client.SvcDeployInput{
 				EnvName: envName,
@@ -147,8 +114,10 @@ var _ = Describe("App Runner", func() {
 
 	Context("run svc show to retrieve the service configuration, resources, and endpoint, then query the service", func() {
 		var (
-			svc          *client.SvcShowOutput
-			svcShowError error
+			svc            *client.SvcShowOutput
+			svcShowError   error
+			backendSvc     *client.SvcShowOutput
+			backendShowErr error
 		)
 
 		BeforeAll(func() {
@@ -157,10 +126,16 @@ var _ = Describe("App Runner", func() {
 				AppName:   appName,
 				Resources: true,
 			})
+			backendSvc, backendShowErr = cli.SvcShow(&client.SvcShowRequest{
+				Name:      beSvcName,
+				AppName:   appName,
+				Resources: true,
+			})
 		})
 
 		It("should not return an error", func() {
 			Expect(svcShowError).NotTo(HaveOccurred())
+			Expect(backendShowErr).NotTo(HaveOccurred())
 		})
 
 		It("should return correct configuration", func() {
@@ -171,6 +146,10 @@ var _ = Describe("App Runner", func() {
 			Expect(svc.Configs[0].CPU).To(Equal("1024"))
 			Expect(svc.Configs[0].Memory).To(Equal("2048"))
 			Expect(svc.Configs[0].Port).To(Equal("80"))
+
+			Expect(backendSvc.SvcName).To(Equal(beSvcName))
+			Expect(backendSvc.AppName).To(Equal(appName))
+			Expect(backendSvc.Configs[0].Environment).To(Equal(envName))
 		})
 
 		It("should return correct environment variables", func() {
@@ -180,14 +159,8 @@ var _ = Describe("App Runner", func() {
 				"COPILOT_ENVIRONMENT_NAME":           envName,
 				"COPILOT_SERVICE_NAME":               feSvcName,
 				"COPILOT_SERVICE_DISCOVERY_ENDPOINT": fmt.Sprintf("%s.%s.local", envName, appName),
-				"S3STORAGE_NAME":                     "", // The value is unknown because it's autogenerated by CloudFormation.
-				"FRONTENDCLUSTER_SECRET_ARN":         "", //same reason as above.
 			}
 			for _, variable := range svc.Variables {
-				if variable.Name == "S3STORAGE_NAME" || variable.Name == "FRONTENDCLUSTER_SECRET_ARN" {
-					Expect(variable.Value).NotTo(BeEmpty())
-					continue
-				}
 				Expect(variable.Value).To(Equal(expectedVars[variable.Name]))
 			}
 		})
@@ -197,7 +170,6 @@ var _ = Describe("App Runner", func() {
 			Expect(svc.Resources[envName]).NotTo(BeNil())
 			expectedTypes := map[string]*countAssertionTracker{
 				"AWS::IAM::Role":                 {3, 0},
-				"AWS::CloudFormation::Stack":     {1, 0},
 				"AWS::AppRunner::Service":        {1, 0},
 				"AWS::EC2::SecurityGroup":        {1, 0},
 				"AWS::EC2::SecurityGroupIngress": {1, 0},
@@ -225,16 +197,19 @@ var _ = Describe("App Runner", func() {
 			route := svc.Routes[0]
 			Expect(route.Environment).To(Equal(envName))
 			Expect(route.URL).NotTo(BeEmpty())
+			Expect(route.Ingress).To(Equal("internet"))
+
+			beRoute := backendSvc.Routes[0]
+			Expect(beRoute.Environment).To(Equal(envName))
+			Expect(beRoute.URL).NotTo(BeEmpty())
+			Expect(beRoute.Ingress).To(Equal("environment"))
+
 			Eventually(func() (int, error) {
 				resp, fetchErr := http.Get(route.URL)
 				return resp.StatusCode, fetchErr
 			}, "30s", "1s").Should(Equal(200))
 			Eventually(func() (int, error) {
-				resp, fetchErr := http.Get(fmt.Sprintf("%s/service-discovery-test/", route.URL))
-				return resp.StatusCode, fetchErr
-			}, "30s", "1s").Should(Equal(200))
-			Eventually(func() (int, error) {
-				resp, fetchErr := http.Get(fmt.Sprintf("%s/db/", route.URL))
+				resp, fetchErr := http.Get(fmt.Sprintf("%s/proxy?url=%s/hello-world", route.URL, beRoute.URL))
 				return resp.StatusCode, fetchErr
 			}, "30s", "1s").Should(Equal(200))
 		})
