@@ -1031,7 +1031,7 @@ func TestInitEnvOpts_Execute(t *testing.T) {
 				)
 				m.cfn.EXPECT().Exists("phonetool-test").Return(false, nil)
 				m.deployer.EXPECT().AddEnvToApp(gomock.Any()).Return(nil)
-				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any()).Return(errors.New("some deploy error"))
+				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any(), gomock.Any()).Return(errors.New("some deploy error"))
 				m.appCFN.EXPECT().GetAppResourcesByRegion(&config.Application{Name: "phonetool"}, "us-west-2").
 					Return(&stack.AppRegionalResources{
 						S3Bucket: "mockBucket",
@@ -1051,7 +1051,7 @@ func TestInitEnvOpts_Execute(t *testing.T) {
 				m.iam.EXPECT().ListRoleTags(gomock.Any()).
 					Return(nil, errors.New("does not exist")).AnyTimes()
 				m.cfn.EXPECT().Exists("phonetool-test").Return(false, nil)
-				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any()).Return(nil)
+				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any(), gomock.Any()).Return(nil)
 				m.deployer.EXPECT().GetEnvironment("phonetool", "test").Return(&config.Environment{
 					App:       "phonetool",
 					Name:      "test",
@@ -1082,7 +1082,7 @@ func TestInitEnvOpts_Execute(t *testing.T) {
 				m.iam.EXPECT().ListRoleTags(gomock.Eq("phonetool-test-CFNExecutionRole")).Return(nil, errors.New("does not exist"))
 				m.iam.EXPECT().ListRoleTags(gomock.Eq("phonetool-test-EnvManagerRole")).Return(nil, errors.New("does not exist"))
 				m.cfn.EXPECT().Exists("phonetool-test").Return(false, nil)
-				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any()).Return(nil)
+				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any(), gomock.Any()).Return(nil)
 				m.deployer.EXPECT().GetEnvironment("phonetool", "test").Return(&config.Environment{
 					AccountID: "1234",
 					Region:    "mars-1",
@@ -1113,7 +1113,7 @@ func TestInitEnvOpts_Execute(t *testing.T) {
 				m.iam.EXPECT().ListRoleTags(gomock.Eq("phonetool-test-CFNExecutionRole")).Return(nil, errors.New("does not exist"))
 				m.iam.EXPECT().ListRoleTags(gomock.Eq("phonetool-test-EnvManagerRole")).Return(nil, errors.New("does not exist"))
 				m.cfn.EXPECT().Exists("phonetool-test").Return(false, nil)
-				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any()).Return(nil)
+				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any(), gomock.Any()).Return(nil)
 				m.deployer.EXPECT().GetEnvironment("phonetool", "test").Return(&config.Environment{
 					AccountID: "1234",
 					Region:    "mars-1",
@@ -1142,15 +1142,19 @@ func TestInitEnvOpts_Execute(t *testing.T) {
 				// Don't attempt to delete any roles since an environment stack already exists.
 				m.iam.EXPECT().ListRoleTags(gomock.Any()).Times(0)
 				m.cfn.EXPECT().Exists("phonetool-test").Return(true, nil)
-				m.deployer.EXPECT().CreateAndRenderEnvironment(&deploy.CreateEnvironmentInput{
-					Name: "test",
-					App: deploy.AppInformation{
-						Name:                "phonetool",
-						AccountPrincipalARN: "some arn",
-					},
-					ArtifactBucketARN:    "arn:aws:s3:::mockBucket",
-					ArtifactBucketKeyARN: "mockKMS",
-				}).Return(&cloudformation.ErrStackAlreadyExists{})
+				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any(), gomock.Any()).DoAndReturn(func(conf deploycfn.StackConfiguration, bucketARN string) error {
+					require.Equal(t, conf, stack.NewBootstrapEnvStackConfig(&stack.EnvConfig{
+						Name: "test",
+						App: deploy.AppInformation{
+							Name:                "phonetool",
+							AccountPrincipalARN: "some arn",
+						},
+						ArtifactBucketARN:    "arn:aws:s3:::mockBucket",
+						ArtifactBucketKeyARN: "mockKMS",
+					}))
+					require.Equal(t, bucketARN, "arn:aws:s3:::mockBucket")
+					return &cloudformation.ErrStackAlreadyExists{}
+				})
 				m.deployer.EXPECT().GetEnvironment("phonetool", "test").Return(&config.Environment{
 					AccountID: "1234",
 					Region:    "mars-1",
@@ -1195,7 +1199,7 @@ func TestInitEnvOpts_Execute(t *testing.T) {
 				m.progress.EXPECT().Start(fmt.Sprintf(fmtDNSDelegationStart, "4567"))
 				m.progress.EXPECT().Stop(log.Ssuccessf(fmtDNSDelegationComplete, "4567"))
 				m.deployer.EXPECT().DelegateDNSPermissions(gomock.Any(), "4567").Return(nil)
-				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any()).Return(nil)
+				m.deployer.EXPECT().CreateAndRenderEnvironment(gomock.Any(), gomock.Any()).Return(nil)
 				m.deployer.EXPECT().GetEnvironment("phonetool", "test").Return(&config.Environment{
 					AccountID: "4567",
 					Region:    "us-west-2",
