@@ -334,7 +334,7 @@ type bucketEmptier interface {
 
 // Interfaces for deploying resources through CloudFormation. Facilitates mocking.
 type environmentDeployer interface {
-	CreateAndRenderEnvironment(env *deploy.CreateEnvironmentInput) error
+	CreateAndRenderEnvironment(conf cloudformation.StackConfiguration, bucketARN string) error
 	DeleteEnvironment(appName, envName, cfnExecRoleARN string) error
 	GetEnvironment(appName, envName string) (*config.Environment, error)
 	EnvironmentTemplate(appName, envName string) (string, error)
@@ -409,10 +409,7 @@ type deployer interface {
 
 type domainHostedZoneGetter interface {
 	DomainHostedZoneID(domainName string) (string, error)
-}
-
-type domainInfoGetter interface {
-	IsRegisteredDomain(domainName string) error
+	ValidateDomainOwnership(domainName string) error
 }
 
 type dockerfileParser interface {
@@ -650,11 +647,12 @@ type runner interface {
 type envDeployer interface {
 	DeployEnvironment(in *clideploy.DeployEnvironmentInput) error
 	Validate(*manifest.Environment) error
-	UploadArtifacts() (map[string]string, error)
+	UploadArtifacts() (*clideploy.UploadEnvArtifactsOutput, error)
 }
 
 type envPackager interface {
 	GenerateCloudFormationTemplate(in *clideploy.DeployEnvironmentInput) (*clideploy.GenerateCloudFormationTemplateOutput, error)
 	Validate(*manifest.Environment) error
-	UploadArtifacts() (map[string]string, error)
+	UploadArtifacts() (*clideploy.UploadEnvArtifactsOutput, error)
+	AddonsTemplate() (string, error)
 }
