@@ -766,11 +766,24 @@ type: Request-Driven Web Service`), nil)
 }
 
 func TestSvcInitOpts_Execute(t *testing.T) {
+	mockEnvironmentManifest := []byte(`name: test
+type: Environment
+network:
+  vpc:
+   id: 'vpc-mockid'
+   subnets:
+      private:
+        - id: 'subnet-1'
+        - id: 'subnet-2'
+        - id: 'subnet-3'
+        - id: 'subnet-4'`)
 	testCases := map[string]struct {
 		mockSvcInit      func(m *mocks.MocksvcInitializer)
 		mockDockerfile   func(m *mocks.MockdockerfileParser)
 		mockDockerEngine func(m *mocks.MockdockerEngine)
 		mockTopicSel     func(m *mocks.MocktopicSelector)
+		mockStore        func(m *mocks.Mockstore)
+		mockEnvDescriber func(m *mocks.MockenvDescriber)
 		inSvcPort        uint16
 		inSvcType        string
 		inSvcName        string
@@ -809,7 +822,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				m.EXPECT().CheckDockerEngineRunning().Return(nil)
 				m.EXPECT().GetPlatform().Return("linux", "amd64", nil)
 			},
-
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 			wantedManifestPath: "manifest/path",
 		},
 		"backend service": {
@@ -836,7 +851,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				m.EXPECT().CheckDockerEngineRunning().Return(nil)
 				m.EXPECT().GetPlatform().Return("linux", "amd64", nil)
 			},
-
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 			wantedManifestPath: "manifest/path",
 		},
 		"doesn't attempt to detect and populate the platform if manifest already exists": {
@@ -865,7 +882,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 					Port: 80,
 				}).Return("manifest/path", nil)
 			},
-
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 			wantedManifestPath: "manifest/path",
 		},
 		"doesn't complain if docker is unavailable": {
@@ -894,6 +913,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 					Port: 80,
 				}).Return("manifest/path", nil)
 			},
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 
 			wantedManifestPath: "manifest/path",
 		},
@@ -926,6 +948,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				m.EXPECT().CheckDockerEngineRunning().Return(nil)
 				m.EXPECT().GetPlatform().Return("windows", "amd64", nil)
 			},
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 
 			wantedManifestPath: "manifest/path",
 		},
@@ -958,6 +983,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				m.EXPECT().CheckDockerEngineRunning().Return(nil)
 				m.EXPECT().GetPlatform().Return("linux", "arm", nil)
 			},
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 
 			wantedManifestPath: "manifest/path",
 		},
@@ -997,6 +1025,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 					},
 				}, nil)
 			},
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 
 			wantedManifestPath: "manifest/path",
 		},
@@ -1019,6 +1050,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				}).Return("manifest/path", nil)
 			},
 			mockDockerfile: func(m *mocks.MockdockerfileParser) {}, // Be sure that no dockerfile parsing happens.
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 
 			wantedManifestPath: "manifest/path",
 		},
@@ -1041,6 +1075,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				}).Return("manifest/path", nil)
 			},
 			mockDockerfile: func(m *mocks.MockdockerfileParser) {}, // Be sure that no dockerfile parsing happens.
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return(nil, nil)
+			},
 
 			wantedManifestPath: "manifest/path",
 		},
@@ -1066,7 +1103,6 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				m.EXPECT().CheckDockerEngineRunning().Return(nil)
 				m.EXPECT().GetPlatform().Return("windows", "amd64", nil)
 			},
-
 			wantedErr: errors.New("redirect docker engine platform: Windows is not supported for App Runner services"),
 		},
 		"failure": {
@@ -1074,10 +1110,80 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				m.EXPECT().CheckDockerEngineRunning().Return(nil)
 				m.EXPECT().GetPlatform().Return("linux", "amd64", nil)
 			},
-			mockSvcInit: func(m *mocks.MocksvcInitializer) {
-				m.EXPECT().Service(gomock.Any()).Return("", errors.New("some error"))
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("").Return(nil, errors.New("some error"))
 			},
-			wantedErr: errors.New("some error"),
+			wantedErr: errors.New("list environments for application : some error"),
+		},
+		"initalize a service in environments with only private subnets": {
+			inAppName:        "sample",
+			inSvcName:        "frontend",
+			inDockerfilePath: "./Dockerfile",
+			inSvcType:        manifest.LoadBalancedWebServiceType,
+
+			inSvcPort: 80,
+
+			mockSvcInit: func(m *mocks.MocksvcInitializer) {
+				m.EXPECT().Service(&initialize.ServiceProps{
+					WorkloadProps: initialize.WorkloadProps{
+						App:            "sample",
+						Name:           "frontend",
+						Type:           "Load Balanced Web Service",
+						DockerfilePath: "./Dockerfile",
+						Platform:       manifest.PlatformArgsOrString{},
+						PrivateOnlyEnvironments: []string{
+							"test",
+						},
+					},
+					Port: 80,
+				}).Return("manifest/path", nil)
+			},
+			mockDockerfile: func(m *mocks.MockdockerfileParser) {
+				m.EXPECT().GetHealthCheck().Return(nil, nil)
+			},
+			mockDockerEngine: func(m *mocks.MockdockerEngine) {
+				m.EXPECT().CheckDockerEngineRunning().Return(nil)
+				m.EXPECT().GetPlatform().Return("linux", "amd64", nil)
+			},
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return([]*config.Environment{
+					{
+						App:  "sample",
+						Name: "test",
+					},
+				}, nil)
+			},
+			mockEnvDescriber: func(m *mocks.MockenvDescriber) {
+				m.EXPECT().Manifest().Return(mockEnvironmentManifest, nil)
+			},
+			wantedManifestPath: "manifest/path",
+		},
+		"error if fail to read the manifest": {
+			inAppName:        "sample",
+			inSvcName:        "frontend",
+			inDockerfilePath: "./Dockerfile",
+			inSvcType:        manifest.LoadBalancedWebServiceType,
+
+			inSvcPort: 80,
+			mockDockerfile: func(m *mocks.MockdockerfileParser) {
+				m.EXPECT().GetHealthCheck().Return(nil, nil)
+			},
+			mockDockerEngine: func(m *mocks.MockdockerEngine) {
+				m.EXPECT().CheckDockerEngineRunning().Return(nil)
+				m.EXPECT().GetPlatform().Return("linux", "amd64", nil)
+			},
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments("sample").Return([]*config.Environment{
+					{
+						App:  "sample",
+						Name: "test",
+					},
+				}, nil)
+			},
+			mockEnvDescriber: func(m *mocks.MockenvDescriber) {
+				m.EXPECT().Manifest().Return(nil, errors.New("failed to read manifest"))
+			},
+			wantedErr: errors.New("read the manifest used to deploy environment test: failed to read manifest"),
 		},
 	}
 
@@ -1091,6 +1197,12 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 			mockDockerfile := mocks.NewMockdockerfileParser(ctrl)
 			mockDockerEngine := mocks.NewMockdockerEngine(ctrl)
 			mockTopicSel := mocks.NewMocktopicSelector(ctrl)
+			mockStore := mocks.NewMockstore(ctrl)
+			mockEnvDescriber := mocks.NewMockenvDescriber(ctrl)
+
+			if tc.mockStore != nil {
+				tc.mockStore(mockStore)
+			}
 
 			if tc.mockSvcInit != nil {
 				tc.mockSvcInit(mockSvcInitializer)
@@ -1101,6 +1213,9 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 			if tc.mockDockerEngine != nil {
 				tc.mockDockerEngine(mockDockerEngine)
 			}
+			if tc.mockEnvDescriber != nil {
+				tc.mockEnvDescriber(mockEnvDescriber)
+			}
 			opts := initSvcOpts{
 				initSvcVars: initSvcVars{
 					initWkldVars: initWkldVars{
@@ -1118,8 +1233,12 @@ func TestSvcInitOpts_Execute(t *testing.T) {
 				},
 				df:             mockDockerfile,
 				dockerEngine:   mockDockerEngine,
+				store:          mockStore,
 				topicSel:       mockTopicSel,
 				manifestExists: tc.inManifestExists,
+				initEnvDescriber: func(string, string) (envDescriber, error) {
+					return mockEnvDescriber, nil
+				},
 			}
 
 			// WHEN
