@@ -38,7 +38,7 @@ type BackendServiceConfig struct {
 	Manifest      *manifest.BackendService
 	RawManifest   []byte // Content of the manifest file without any transformations.
 	RuntimeConfig RuntimeConfig
-	Addons        addons
+	Addons        NestedStackConfigurer
 }
 
 // NewBackendService creates a new BackendService stack from a manifest file.
@@ -148,7 +148,7 @@ func (s *BackendService) Template() (string, error) {
 		SerializedManifest: string(s.rawManifest),
 		EnvVersion:         s.rc.EnvVersion,
 
-		Variables:          s.manifest.BackendServiceConfig.Variables,
+		Variables:          convertEnvVars(s.manifest.BackendServiceConfig.Variables),
 		Secrets:            convertSecrets(s.manifest.BackendServiceConfig.Secrets),
 		Aliases:            aliases,
 		HTTPSListener:      s.httpsEnabled,
