@@ -24,16 +24,20 @@ import (
 
 func TestEnvStack_Template(t *testing.T) {
 	testCases := map[string]struct {
-		input          *deploy.CreateEnvironmentInput
+		input          *stack.EnvConfig
 		wantedFileName string
 	}{
-		"generate template with embedded manifest file with container insights and imported certificates and advanced access logs": {
-			input: func() *deploy.CreateEnvironmentInput {
+		"generate template with embedded manifest file with container insights and cloudfront and advanced access logs": {
+			input: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 # Create the public ALB with certificates attached.
 cdn:
   certificate: viewer-cert
+  static_assets:
+    location: cf-s3-ecs-demo-bucket.s3.us-west-2.amazonaws.com
+    alias: example.com
+    path: static/*
 http:
   public:
     ingress:
@@ -56,7 +60,7 @@ observability:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -77,10 +81,10 @@ observability:
 					RawMft: []byte(rawMft),
 				}
 			}(),
-			wantedFileName: "template-with-imported-certs-observability.yml",
+			wantedFileName: "template-with-cloudfront-observability.yml",
 		},
 		"generate template with default access logs": {
-			input: func() *deploy.CreateEnvironmentInput {
+			input: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 http:
@@ -89,7 +93,7 @@ http:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -110,7 +114,7 @@ http:
 			wantedFileName: "template-with-default-access-log-config.yml",
 		},
 		"generate template with embedded manifest file with custom security groups rules added by the customer": {
-			input: func() *deploy.CreateEnvironmentInput {
+			input: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 # Create the public ALB with certificates attached.
@@ -138,7 +142,7 @@ network:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -160,7 +164,7 @@ network:
 			wantedFileName: "template-with-custom-security-group.yml",
 		},
 		"generate template with embedded manifest file with imported certificates and SSL Policy and empty security groups rules added by the customer": {
-			input: func() *deploy.CreateEnvironmentInput {
+			input: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 # Create the public ALB with certificates attached.
@@ -178,7 +182,7 @@ security_group:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -200,13 +204,13 @@ security_group:
 			wantedFileName: "template-with-imported-certs-sslpolicy-custom-empty-security-group.yml",
 		},
 		"generate template with custom resources": {
-			input: func() *deploy.CreateEnvironmentInput {
+			input: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment`
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -227,7 +231,7 @@ type: Environment`
 			wantedFileName: "template-with-basic-manifest.yml",
 		},
 		"generate template with default vpc and flowlogs is on": {
-			input: func() *deploy.CreateEnvironmentInput {
+			input: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 network:
@@ -237,7 +241,7 @@ network:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -258,7 +262,7 @@ network:
 			wantedFileName: "template-with-defaultvpc-flowlogs.yml",
 		},
 		"generate template with imported vpc and flowlogs is on": {
-			input: func() *deploy.CreateEnvironmentInput {
+			input: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 network:
@@ -275,7 +279,7 @@ network:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -324,11 +328,11 @@ network:
 
 func TestEnvStack_Regression(t *testing.T) {
 	testCases := map[string]struct {
-		originalManifest *deploy.CreateEnvironmentInput
-		newManifest      *deploy.CreateEnvironmentInput
+		originalManifest *stack.EnvConfig
+		newManifest      *stack.EnvConfig
 	}{
 		"should produce the same template after migrating load balancer ingress fields": {
-			originalManifest: func() *deploy.CreateEnvironmentInput {
+			originalManifest: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 # Create the public ALB with certificates attached.
@@ -355,7 +359,7 @@ observability:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
@@ -375,7 +379,7 @@ observability:
 					RawMft: []byte(rawMft),
 				}
 			}(),
-			newManifest: func() *deploy.CreateEnvironmentInput {
+			newManifest: func() *stack.EnvConfig {
 				rawMft := `name: test
 type: Environment
 # Create the public ALB with certificates attached.
@@ -399,7 +403,7 @@ observability:
 				var mft manifest.Environment
 				err := yaml.Unmarshal([]byte(rawMft), &mft)
 				require.NoError(t, err)
-				return &deploy.CreateEnvironmentInput{
+				return &stack.EnvConfig{
 					Version: "1.x",
 					App: deploy.AppInformation{
 						AccountPrincipalARN: "arn:aws:iam::000000000:root",
