@@ -21,81 +21,176 @@ import (
 )
 
 func TestEnv_Template(t *testing.T) {
-	testCases := map[string]struct {
-		mockDependencies func(ctrl *gomock.Controller, e *EnvStackConfig)
-		expectedOutput   string
-		want             error
-	}{
-		"should return template body when present": {
-			mockDependencies: func(ctrl *gomock.Controller, e *EnvStackConfig) {
-				m := mocks.NewMockenvReadParser(ctrl)
-				m.EXPECT().ParseEnv(gomock.Any()).DoAndReturn(func(data *template.EnvOpts) (*template.Content, error) {
-					require.Equal(t, &template.EnvOpts{
-						AppName: "project",
-						EnvName: "env",
-						VPCConfig: template.VPCConfig{
-							Imported: nil,
-							Managed: template.ManagedVPC{
-								CIDR:               DefaultVPCCIDR,
-								PrivateSubnetCIDRs: DefaultPrivateSubnetCIDRs,
-								PublicSubnetCIDRs:  DefaultPublicSubnetCIDRs,
-							},
-							SecurityGroupConfig: nil,
-							FlowLogs:            nil,
-						},
-						LatestVersion: deploy.LatestEnvTemplateVersion,
-						CustomResources: map[string]template.S3ObjectLocation{
-							"CertificateValidationFunction": {
-								Bucket: "mockbucket",
-								Key:    "mockkey1",
-							},
-							"DNSDelegationFunction": {
-								Bucket: "mockbucket",
-								Key:    "mockkey2",
-							},
-							"CustomDomainFunction": {
-								Bucket: "mockbucket",
-								Key:    "mockkey4",
-							},
-						},
-						Telemetry: &template.Telemetry{
-							EnableContainerInsights: false,
-						},
+	t.Run("error parsing template", func(t *testing.T) {
+		ctrl := gomock.NewController(t)
+		defer ctrl.Finish()
 
-						SerializedManifest: "name: env\ntype: Environment\n",
-						ForceUpdateID:      "mockPreviousForceUpdateID",
-					}, data)
-					return &template.Content{Buffer: bytes.NewBufferString("mockTemplate")}, nil
-				})
-				e.parser = m
-			},
-			expectedOutput: mockTemplate,
-		},
-	}
+		// GIVEN
+		inEnvConfig := mockDeployEnvironmentInput()
+		mockParser := mocks.NewMockenvReadParser(ctrl)
 
-	for name, tc := range testCases {
-		t.Run(name, func(t *testing.T) {
-			// GIVEN
-			ctrl := gomock.NewController(t)
-			defer ctrl.Finish()
-			envStack := &EnvStackConfig{
-				in:                mockDeployEnvironmentInput(),
-				lastForceUpdateID: "mockPreviousForceUpdateID",
-			}
-			tc.mockDependencies(ctrl, envStack)
+		// EXPECT
+		mockParser.EXPECT().ParseEnv(gomock.Any()).Return(nil, errors.New("some error"))
 
-			// WHEN
-			got, err := envStack.Template()
+		// WHEN
+		envStack := &Env{
+			in:                inEnvConfig,
+			lastForceUpdateID: "mockPreviousForceUpdateID",
+			parser:            mockParser,
+		}
+		_, err := envStack.Template()
 
-			// THEN
-			if tc.want != nil {
-				require.EqualError(t, tc.want, err.Error())
-			} else {
-				require.NoError(t, err)
-				require.Equal(t, tc.expectedOutput, got)
-			}
+		// THEN
+		require.EqualError(t, errors.New("some error"), err.Error())
+	})
+	t.Run("error parsing addons extra parameters", func(t *testing.T) {
+		ctrl := gomock.NewController(t)
+		defer ctrl.Finish()
+
+		// GIVEN
+		inEnvConfig := mockDeployEnvironmentInput()
+		mockAddonsConfig := mocks.NewMockNestedStackConfigurer(ctrl)
+		mockParser := mocks.NewMockenvReadParser(ctrl)
+		inEnvConfig.Addons = &Addons{
+			S3ObjectURL: "mockAddonsURL",
+			Stack:       mockAddonsConfig,
+		}
+		// EXPECT
+		mockAddonsConfig.EXPECT().Parameters().Return("", errors.New("some error"))
+
+		// WHEN
+		envStack := &Env{
+			in:                inEnvConfig,
+			lastForceUpdateID: "mockPreviousForceUpdateID",
+			parser:            mockParser,
+		}
+		_, err := envStack.Template()
+
+		// THEN
+		require.EqualError(t, errors.New("parse extra parameters for environment addons: some error"), err.Error())
+	})
+	t.Run("should contain addons information when addons are present", func(t *testing.T) {
+		ctrl := gomock.NewController(t)
+		defer ctrl.Finish()
+
+		// GIVEN
+		inEnvConfig := mockDeployEnvironmentInput()
+		mockAddonsConfig := mocks.NewMockNestedStackConfigurer(ctrl)
+		mockParser := mocks.NewMockenvReadParser(ctrl)
+		inEnvConfig.Addons = &Addons{
+			S3ObjectURL: "mockAddonsURL",
+			Stack:       mockAddonsConfig,
+		}
+
+		// EXPECT
+		mockAddonsConfig.EXPECT().Parameters().Return("mockAddonsExtraParameters", nil)
+		mockParser.EXPECT().ParseEnv(gomock.Any()).DoAndReturn(func(data *template.EnvOpts) (*template.Content, error) {
+			require.Equal(t, &template.Addons{
+				URL:         "mockAddonsURL",
+				ExtraParams: "mockAddonsExtraParameters",
+			}, data.Addons)
+			return &template.Content{Buffer: bytes.NewBufferString("mockTemplate")}, nil
 		})
-	}
+
+		// WHEN
+		envStack := &Env{
+			in:                inEnvConfig,
+			lastForceUpdateID: "mockPreviousForceUpdateID",
+			parser:            mockParser,
+		}
+		got, err := envStack.Template()
+
+		// THEN
+		require.NoError(t, err)
+		require.Equal(t, mockTemplate, got)
+	})
+	t.Run("should use new force update ID when asked", func(t *testing.T) {
+		ctrl := gomock.NewController(t)
+		defer ctrl.Finish()
+
+		// GIVEN
+		inEnvConfig := mockDeployEnvironmentInput()
+		inEnvConfig.ForceUpdate = true
+		mockParser := mocks.NewMockenvReadParser(ctrl)
+
+		// EXPECT
+		mockParser.EXPECT().ParseEnv(gomock.Any()).DoAndReturn(func(data *template.EnvOpts) (*template.Content, error) {
+			require.NotEqual(t, "mockPreviousForceUpdateID", data.ForceUpdateID)
+			return &template.Content{Buffer: bytes.NewBufferString("mockTemplate")}, nil
+		})
+
+		// WHEN
+		envStack := &Env{
+			in:                inEnvConfig,
+			lastForceUpdateID: "mockPreviousForceUpdateID",
+			parser:            mockParser,
+		}
+		got, err := envStack.Template()
+
+		// THEN
+		require.NoError(t, err)
+		require.Equal(t, mockTemplate, got)
+	})
+	t.Run("should return template body when present", func(t *testing.T) {
+		ctrl := gomock.NewController(t)
+		defer ctrl.Finish()
+
+		// GIVEN
+		inEnvConfig := mockDeployEnvironmentInput()
+		mockParser := mocks.NewMockenvReadParser(ctrl)
+
+		// EXPECT
+		mockParser.EXPECT().ParseEnv(gomock.Any()).DoAndReturn(func(data *template.EnvOpts) (*template.Content, error) {
+			require.Equal(t, &template.EnvOpts{
+				AppName: "project",
+				EnvName: "env",
+				VPCConfig: template.VPCConfig{
+					Imported: nil,
+					Managed: template.ManagedVPC{
+						CIDR:               DefaultVPCCIDR,
+						PrivateSubnetCIDRs: DefaultPrivateSubnetCIDRs,
+						PublicSubnetCIDRs:  DefaultPublicSubnetCIDRs,
+					},
+					SecurityGroupConfig: nil,
+					FlowLogs:            nil,
+				},
+				LatestVersion: deploy.LatestEnvTemplateVersion,
+				CustomResources: map[string]template.S3ObjectLocation{
+					"CertificateValidationFunction": {
+						Bucket: "mockbucket",
+						Key:    "mockkey1",
+					},
+					"DNSDelegationFunction": {
+						Bucket: "mockbucket",
+						Key:    "mockkey2",
+					},
+					"CustomDomainFunction": {
+						Bucket: "mockbucket",
+						Key:    "mockkey4",
+					},
+				},
+				Telemetry: &template.Telemetry{
+					EnableContainerInsights: false,
+				},
+
+				SerializedManifest: "name: env\ntype: Environment\n",
+				ForceUpdateID:      "mockPreviousForceUpdateID",
+			}, data)
+			return &template.Content{Buffer: bytes.NewBufferString("mockTemplate")}, nil
+		})
+
+		// WHEN
+		envStack := &Env{
+			in:                inEnvConfig,
+			lastForceUpdateID: "mockPreviousForceUpdateID",
+			parser:            mockParser,
+		}
+		got, err := envStack.Template()
+
+		// THEN
+		require.NoError(t, err)
+		require.Equal(t, mockTemplate, got)
+	})
 }
 
 func TestEnv_Parameters(t *testing.T) {
@@ -105,7 +200,7 @@ func TestEnv_Parameters(t *testing.T) {
 	deploymentInputWithPrivateDNS := mockDeployEnvironmentInput()
 	deploymentInputWithPrivateDNS.Mft.HTTPConfig.Private.Certificates = []string{"arn:aws:acm:us-east-1:123456789012:certificate/12345678-1234-1234-1234-123456789012"}
 	testCases := map[string]struct {
-		input     *deploy.CreateEnvironmentInput
+		input     *EnvConfig
 		oldParams []*cloudformation.Parameter
 		want      []*cloudformation.Parameter
 	}{
@@ -867,7 +962,7 @@ func TestEnv_Parameters(t *testing.T) {
 
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
-			env := &EnvStackConfig{
+			env := &Env{
 				in:         tc.input,
 				prevParams: tc.oldParams,
 			}
@@ -879,8 +974,8 @@ func TestEnv_Parameters(t *testing.T) {
 }
 
 func TestEnv_Tags(t *testing.T) {
-	env := &EnvStackConfig{
-		in: &deploy.CreateEnvironmentInput{
+	env := &Env{
+		in: &EnvConfig{
 			Name: "env",
 			App: deploy.AppInformation{
 				Name: "project",
@@ -910,7 +1005,7 @@ func TestEnv_Tags(t *testing.T) {
 
 func TestStackName(t *testing.T) {
 	deploymentInput := mockDeployEnvironmentInput()
-	env := &EnvStackConfig{
+	env := &Env{
 		in: deploymentInput,
 	}
 	require.Equal(t, fmt.Sprintf("%s-%s", deploymentInput.App.Name, deploymentInput.Name), env.StackName())
@@ -918,20 +1013,20 @@ func TestStackName(t *testing.T) {
 
 func TestBootstrapEnv_Template(t *testing.T) {
 	testCases := map[string]struct {
-		in             *deploy.CreateEnvironmentInput
+		in             *EnvConfig
 		setupMock      func(m *mocks.MockenvReadParser)
 		expectedOutput string
 		wantedError    error
 	}{
 		"error parsing the template": {
-			in: &deploy.CreateEnvironmentInput{},
+			in: &EnvConfig{},
 			setupMock: func(m *mocks.MockenvReadParser) {
 				m.EXPECT().ParseEnvBootstrap(gomock.Any(), gomock.Any()).Return(nil, errors.New("some error"))
 			},
 			wantedError: errors.New("some error"),
 		},
 		"should return template body when present": {
-			in: &deploy.CreateEnvironmentInput{
+			in: &EnvConfig{
 				ArtifactBucketARN:    "mockBucketARN",
 				ArtifactBucketKeyARN: "mockBucketKeyARN",
 			},
@@ -956,7 +1051,7 @@ func TestBootstrapEnv_Template(t *testing.T) {
 
 			mockParser := mocks.NewMockenvReadParser(ctrl)
 			tc.setupMock(mockParser)
-			bootstrapStack := &BootstrapEnvStackConfig{
+			bootstrapStack := &BootstrapEnv{
 				in:     tc.in,
 				parser: mockParser,
 			}
@@ -977,11 +1072,11 @@ func TestBootstrapEnv_Template(t *testing.T) {
 
 func TestBootstrapEnv_Parameters(t *testing.T) {
 	testCases := map[string]struct {
-		input *deploy.CreateEnvironmentInput
+		input *EnvConfig
 		want  []*cloudformation.Parameter
 	}{
 		"returns correct parameters": {
-			input: &deploy.CreateEnvironmentInput{
+			input: &EnvConfig{
 				App: deploy.AppInformation{
 					Name:                "mockApp",
 					AccountPrincipalARN: "mockAccountPrincipalARN",
@@ -1007,7 +1102,7 @@ func TestBootstrapEnv_Parameters(t *testing.T) {
 
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
-			bootstrap := &BootstrapEnvStackConfig{
+			bootstrap := &BootstrapEnv{
 				in: tc.input,
 			}
 			params, err := bootstrap.Parameters()
@@ -1018,8 +1113,8 @@ func TestBootstrapEnv_Parameters(t *testing.T) {
 }
 
 func TestBootstrapEnv_Tags(t *testing.T) {
-	bootstrap := &BootstrapEnvStackConfig{
-		in: &deploy.CreateEnvironmentInput{
+	bootstrap := &BootstrapEnv{
+		in: &EnvConfig{
 			Name: "env",
 			App: deploy.AppInformation{
 				Name: "project",
@@ -1048,8 +1143,8 @@ func TestBootstrapEnv_Tags(t *testing.T) {
 }
 
 func TestBootstrapEnv_StackName(t *testing.T) {
-	bootstrap := &BootstrapEnvStackConfig{
-		in: &deploy.CreateEnvironmentInput{
+	bootstrap := &BootstrapEnv{
+		in: &EnvConfig{
 			App: deploy.AppInformation{
 				Name: "mockApp",
 			},
@@ -1088,10 +1183,10 @@ func TestBootstrapEnv_ToEnv(t *testing.T) {
 
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
-			envStack := &BootstrapEnvStackConfig{
+			envStack := &BootstrapEnv{
 				in: mockDeployInput,
 			}
-			got, err := envStack.ToEnv(tc.mockStack)
+			got, err := envStack.ToEnvMetadata(tc.mockStack)
 
 			if tc.want != nil {
 				require.EqualError(t, tc.want, err.Error())
@@ -1119,8 +1214,8 @@ func mockEnvironmentStack(stackArn, managerRoleARN, executionRoleARN string) *cl
 	}
 }
 
-func mockDeployEnvironmentInput() *deploy.CreateEnvironmentInput {
-	return &deploy.CreateEnvironmentInput{
+func mockDeployEnvironmentInput() *EnvConfig {
+	return &EnvConfig{
 		Name: "env",
 		App: deploy.AppInformation{
 			Name:                "project",
