@@ -77,6 +77,7 @@ func TestNewHTTPLoadBalancedWebService(t *testing.T) {
 						},
 					},
 				},
+				Environments: map[string]*LoadBalancedWebServiceConfig{},
 			},
 		},
 		"overrides default settings when optional configuration is provided": {
@@ -84,6 +85,9 @@ func TestNewHTTPLoadBalancedWebService(t *testing.T) {
 				WorkloadProps: &WorkloadProps{
 					Name:       "subscribers",
 					Dockerfile: "./subscribers/Dockerfile",
+					PrivateOnlyEnvironments: []string{
+						"metrics",
+					},
 				},
 				Path: "/",
 				Port: 80,
@@ -153,6 +157,17 @@ func TestNewHTTPLoadBalancedWebService(t *testing.T) {
 						},
 					},
 				},
+				Environments: map[string]*LoadBalancedWebServiceConfig{
+					"metrics": {
+						Network: NetworkConfig{
+							VPC: vpcConfig{
+								Placement: PlacementArgOrString{
+									PlacementString: placementStringP(PrivateSubnetPlacement),
+								},
+							},
+						},
+					},
+				},
 			},
 		},
 	}
@@ -265,6 +280,20 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 						Count: Count{
 							Value: aws.Int(1),
 						},
+						Variables: map[string]Variable{
+							"VAR1": {
+								stringOrFromCFN{
+									Plain: stringP("var1"),
+								},
+							},
+							"VAR2": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{
+										Name: stringP("import-var2"),
+									},
+								},
+							},
+						},
 						Storage: Storage{
 							Volumes: map[string]*Volume{
 								"myEFSVolume": {
@@ -317,6 +346,20 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 						Count: Count{
 							Value: aws.Int(1),
 						},
+						Variables: map[string]Variable{
+							"VAR1": {
+								stringOrFromCFN{
+									Plain: stringP("var1"),
+								},
+							},
+							"VAR2": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{
+										Name: stringP("import-var2"),
+									},
+								},
+							},
+						},
 						Storage: Storage{
 							Volumes: map[string]*Volume{
 								"myEFSVolume": {
@@ -369,13 +412,43 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 						Count: Count{
 							Value: aws.Int(1),
 						},
-						Variables: map[string]string{
-							"LOG_LEVEL":      "DEBUG",
-							"DDB_TABLE_NAME": "awards",
+						Variables: map[string]Variable{
+							"LOG_LEVEL": {
+								stringOrFromCFN{
+									Plain: stringP("DEBUG"),
+								},
+							},
+							"S3_TABLE_NAME": {
+								stringOrFromCFN{
+									Plain: stringP("doggo"),
+								},
+							},
+							"RDS_TABLE_NAME": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{
+										Name: stringP("duckling"),
+									},
+								},
+							},
+							"DDB_TABLE_NAME": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{
+										Name: stringP("awards"),
+									},
+								},
+							},
 						},
 						Secrets: map[string]Secret{
-							"GITHUB_TOKEN": {from: aws.String("1111")},
-							"TWILIO_TOKEN": {from: aws.String("1111")},
+							"GITHUB_TOKEN": {
+								from: stringOrFromCFN{
+									Plain: aws.String("1111"),
+								},
+							},
+							"TWILIO_TOKEN": {
+								from: stringOrFromCFN{
+									Plain: aws.String("1111"),
+								},
+							},
 						},
 						Storage: Storage{
 							Volumes: map[string]*Volume{
@@ -413,7 +486,9 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 								PlacementString: placementStringP(PublicSubnetPlacement),
 							},
 							SecurityGroups: SecurityGroupsIDsOrConfig{
-								IDs: []string{"sg-123"},
+								IDs: []stringOrFromCFN{{
+									Plain: aws.String("sg-123"),
+								}},
 							},
 						},
 					},
@@ -442,8 +517,25 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 							Count: Count{
 								Value: aws.Int(0),
 							},
-							Variables: map[string]string{
-								"DDB_TABLE_NAME": "awards-prod",
+							Variables: map[string]Variable{
+								"LOG_LEVEL": {
+									stringOrFromCFN{
+										Plain: stringP("ERROR"),
+									},
+								},
+								"S3_TABLE_NAME": {
+									stringOrFromCFN{
+										FromCFN: fromCFN{Name: stringP("prod-doggo")},
+									},
+								},
+								"RDS_TABLE_NAME": {
+									stringOrFromCFN{Plain: stringP("duckling-prod")},
+								},
+								"DDB_TABLE_NAME": {
+									stringOrFromCFN{
+										FromCFN: fromCFN{Name: stringP("awards-prod")},
+									},
+								},
 							},
 							Storage: Storage{
 								Volumes: map[string]*Volume{
@@ -476,13 +568,24 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 						},
 						Logging: Logging{
 							SecretOptions: map[string]Secret{
-								"FOO": {from: aws.String("BAR")},
+								"FOO": {
+									from: stringOrFromCFN{
+										Plain: aws.String("BAR"),
+									},
+								},
 							},
 						},
 						Network: NetworkConfig{
 							VPC: vpcConfig{
 								SecurityGroups: SecurityGroupsIDsOrConfig{
-									IDs: []string{"sg-456", "sg-789"},
+									IDs: []stringOrFromCFN{
+										{
+											Plain: aws.String("sg-456"),
+										},
+										{
+											Plain: aws.String("sg-789"),
+										},
+									},
 								},
 							},
 						},
@@ -524,13 +627,39 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 						Count: Count{
 							Value: aws.Int(0),
 						},
-						Variables: map[string]string{
-							"LOG_LEVEL":      "DEBUG",
-							"DDB_TABLE_NAME": "awards-prod",
+						Variables: map[string]Variable{
+							"LOG_LEVEL": {
+								stringOrFromCFN{
+									Plain: stringP("ERROR"),
+								},
+							},
+							"S3_TABLE_NAME": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{Name: stringP("prod-doggo")},
+								},
+							},
+							"RDS_TABLE_NAME": {
+								stringOrFromCFN{
+									Plain: stringP("duckling-prod"),
+								},
+							},
+							"DDB_TABLE_NAME": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{Name: stringP("awards-prod")},
+								},
+							},
 						},
 						Secrets: map[string]Secret{
-							"GITHUB_TOKEN": {from: aws.String("1111")},
-							"TWILIO_TOKEN": {from: aws.String("1111")},
+							"GITHUB_TOKEN": {
+								from: stringOrFromCFN{
+									Plain: aws.String("1111"),
+								},
+							},
+							"TWILIO_TOKEN": {
+								from: stringOrFromCFN{
+									Plain: aws.String("1111"),
+								},
+							},
 						},
 						Storage: Storage{
 							Volumes: map[string]*Volume{
@@ -571,7 +700,11 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 					Logging: Logging{
 						ConfigFile: aws.String("mockConfigFile"),
 						SecretOptions: map[string]Secret{
-							"FOO": {from: aws.String("BAR")},
+							"FOO": {
+								from: stringOrFromCFN{
+									Plain: aws.String("BAR"),
+								},
+							},
 						},
 					},
 					Network: NetworkConfig{
@@ -580,7 +713,14 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 								PlacementString: placementStringP(PublicSubnetPlacement),
 							},
 							SecurityGroups: SecurityGroupsIDsOrConfig{
-								IDs: []string{"sg-456", "sg-789"},
+								IDs: []stringOrFromCFN{
+									{
+										Plain: aws.String("sg-456"),
+									},
+									{
+										Plain: aws.String("sg-789"),
+									},
+								},
 							},
 						},
 					},
@@ -597,6 +737,18 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 								CPU:   mockConfig,
 							},
 						},
+						Variables: map[string]Variable{
+							"VAR1": {
+								stringOrFromCFN{
+									Plain: stringP("var1"),
+								},
+							},
+							"VAR2": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{Name: stringP("import-var2")},
+								},
+							},
+						},
 					},
 					ImageOverride: ImageOverride{
 						Command: CommandOverride{
@@ -623,6 +775,18 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 								CPU:   mockConfig,
 							},
 						},
+						Variables: map[string]Variable{
+							"VAR1": {
+								stringOrFromCFN{
+									Plain: stringP("var1"),
+								},
+							},
+							"VAR2": {
+								stringOrFromCFN{
+									FromCFN: fromCFN{Name: stringP("import-var2")},
+								},
+							},
+						},
 					},
 					ImageOverride: ImageOverride{
 						Command: CommandOverride{
@@ -655,7 +819,14 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 								PlacementString: placementStringP(PublicSubnetPlacement),
 							},
 							SecurityGroups: SecurityGroupsIDsOrConfig{
-								IDs: []string{"sg-456", "sg-789"},
+								IDs: []stringOrFromCFN{
+									{
+										Plain: aws.String("sg-456"),
+									},
+									{
+										Plain: aws.String("sg-789"),
+									},
+								},
 							},
 						},
 					},
@@ -683,7 +854,14 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 								PlacementString: placementStringP(PublicSubnetPlacement),
 							},
 							SecurityGroups: SecurityGroupsIDsOrConfig{
-								IDs: []string{"sg-456", "sg-789"},
+								IDs: []stringOrFromCFN{
+									{
+										Plain: aws.String("sg-456"),
+									},
+									{
+										Plain: aws.String("sg-789"),
+									},
+								},
 							},
 						},
 					},
@@ -700,7 +878,14 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 							},
 							SecurityGroups: SecurityGroupsIDsOrConfig{
 								AdvancedConfig: SecurityGroupsConfig{
-									SecurityGroups: []string{"sg-535", "sg-789"},
+									SecurityGroups: []stringOrFromCFN{
+										{
+											Plain: aws.String("sg-535"),
+										},
+										{
+											Plain: aws.String("sg-789"),
+										},
+									},
 								},
 							},
 						},
@@ -712,8 +897,15 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 							VPC: vpcConfig{
 								SecurityGroups: SecurityGroupsIDsOrConfig{
 									AdvancedConfig: SecurityGroupsConfig{
-										SecurityGroups: []string{"sg-456", "sg-700"},
-										DenyDefault:    aws.Bool(true),
+										SecurityGroups: []stringOrFromCFN{
+											{
+												Plain: aws.String("sg-456"),
+											},
+											{
+												Plain: aws.String("sg-700"),
+											},
+										},
+										DenyDefault: aws.Bool(true),
 									},
 								},
 							},
@@ -732,8 +924,15 @@ func TestLoadBalancedWebService_ApplyEnv(t *testing.T) {
 							},
 							SecurityGroups: SecurityGroupsIDsOrConfig{
 								AdvancedConfig: SecurityGroupsConfig{
-									SecurityGroups: []string{"sg-456", "sg-700"},
-									DenyDefault:    aws.Bool(true),
+									SecurityGroups: []stringOrFromCFN{
+										{
+											Plain: aws.String("sg-456"),
+										},
+										{
+											Plain: aws.String("sg-700"),
+										},
+									},
+									DenyDefault: aws.Bool(true),
 								},
 							},
 						},
@@ -1554,3 +1753,507 @@ func TestLoadBalancedWebService_RequiredEnvironmentFeatures(t *testing.T) {
 		})
 	}
 }
+
+func TestLoadBalancedWebService_ExposedPorts(t *testing.T) {
+	testCases := map[string]struct {
+		mft                *LoadBalancedWebService
+		wantedExposedPorts []ExposedPort
+	}{
+		"expose new sidecar container port through alb target_port and target_container": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:            aws.String("/"),
+							TargetContainer: aws.String("xray"),
+							TargetPort:      aws.Uint16(81),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("2000"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          2000,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"expose new primary container port through alb target_port": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:       aws.String("/"),
+							TargetPort: aws.Uint16(81),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("2000"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          2000,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"expose new primary container port through alb target_port and target_container": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:            aws.String("/"),
+							TargetContainer: aws.String("frontend"),
+							TargetPort:      aws.Uint16(81),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("2000"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          2000,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"expose sidecar container port through alb target_port": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:            aws.String("/"),
+							TargetContainer: aws.String("xray"),
+							TargetPort:      aws.Uint16(81),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"reference existing sidecar container port through alb target_port": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:       aws.String("/"),
+							TargetPort: aws.Uint16(81),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("81"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"reference existing primary container port through alb target_port": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:       aws.String("/"),
+							TargetPort: aws.Uint16(80),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("81"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"ALB and NLB exposes the same additional port on the main container": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:       aws.String("/"),
+							TargetPort: aws.Uint16(81),
+						},
+					},
+					NLBConfig: NetworkLoadBalancerConfiguration{
+						Port:       aws.String("85"),
+						TargetPort: aws.Int(81),
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("2000"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          2000,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"ALB and NLB exposes two different ports on the main container": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							TargetPort: aws.Uint16(81),
+						},
+					},
+					NLBConfig: NetworkLoadBalancerConfiguration{
+						Port: aws.String("82"),
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("2000"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          81,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          82,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          2000,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"expose new primary container port through NLB config": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(80),
+						},
+					},
+					NLBConfig: NetworkLoadBalancerConfiguration{
+						Port: aws.String("82"),
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("2000"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          82,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          2000,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"alb and nlb pointing to the same primary container port": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(8080),
+						},
+					},
+					NLBConfig: NetworkLoadBalancerConfiguration{
+						Port: aws.String("8080"),
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:       aws.String("/"),
+							TargetPort: aws.Uint16(8080),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("80"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          8080,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+			},
+		},
+		"alb and nlb exposing new ports of the main and sidecar containers": {
+			mft: &LoadBalancedWebService{
+				Workload: Workload{
+					Name: aws.String("frontend"),
+				},
+				LoadBalancedWebServiceConfig: LoadBalancedWebServiceConfig{
+					ImageConfig: ImageWithPortAndHealthcheck{
+						ImageWithPort: ImageWithPort{
+							Port: aws.Uint16(8080),
+						},
+					},
+					NLBConfig: NetworkLoadBalancerConfiguration{
+						Port:            aws.String("8082/tcp"),
+						TargetContainer: aws.String("xray"),
+					},
+					RoutingRule: RoutingRuleConfigOrBool{
+						RoutingRuleConfiguration: RoutingRuleConfiguration{
+							Path:       aws.String("/"),
+							TargetPort: aws.Uint16(8081),
+						},
+					},
+					Sidecars: map[string]*SidecarConfig{
+						"xray": {
+							Port:       aws.String("80"),
+							Image:      aws.String("123456789012.dkr.ecr.us-east-2.amazonaws.com/xray-daemon"),
+							CredsParam: aws.String("some arn"),
+						},
+					},
+				},
+			},
+			wantedExposedPorts: []ExposedPort{
+				{
+					Port:          80,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          8080,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          8081,
+					ContainerName: "frontend",
+					Protocol:      "tcp",
+				},
+				{
+					Port:          8082,
+					ContainerName: "xray",
+					Protocol:      "tcp",
+				},
+			},
+		},
+	}
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			// WHEN
+			actual, err := tc.mft.ExposedPorts()
+
+			// THEN
+			require.NoError(t, err)
+			require.Equal(t, tc.wantedExposedPorts, actual)
+		})
+	}
+}
