@@ -129,11 +129,19 @@ environments:
 							ExecuteCommand: ExecuteCommand{
 								Enable: aws.Bool(true),
 							},
-							Variables: map[string]string{
-								"LOG_LEVEL": "WARN",
+							Variables: map[string]Variable{
+								"LOG_LEVEL": {
+									stringOrFromCFN{
+										Plain: stringP("WARN"),
+									},
+								},
 							},
 							Secrets: map[string]Secret{
-								"DB_PASSWORD": {from: aws.String("MYSQL_DB_PASSWORD")},
+								"DB_PASSWORD": {
+									from: stringOrFromCFN{
+										Plain: aws.String("MYSQL_DB_PASSWORD"),
+									},
+								},
 							},
 						},
 						Sidecars: map[string]*SidecarConfig{
@@ -152,7 +160,11 @@ environments:
 							EnableMetadata: aws.Bool(false),
 							ConfigFile:     aws.String("/extra.conf"),
 							SecretOptions: map[string]Secret{
-								"LOG_TOKEN": {from: aws.String("LOG_TOKEN")},
+								"LOG_TOKEN": {
+									from: stringOrFromCFN{
+										Plain: aws.String("LOG_TOKEN"),
+									},
+								},
 							},
 						},
 						Network: NetworkConfig{
@@ -276,7 +288,11 @@ secrets:
 								Enable: aws.Bool(false),
 							},
 							Secrets: map[string]Secret{
-								"API_TOKEN": {from: aws.String("SUBS_API_TOKEN")},
+								"API_TOKEN": {
+									from: stringOrFromCFN{
+										Plain: aws.String("SUBS_API_TOKEN"),
+									},
+								},
 							},
 						},
 						Network: NetworkConfig{
@@ -287,6 +303,7 @@ secrets:
 							},
 						},
 					},
+					Environments: map[string]*BackendServiceConfig{},
 				}
 				require.Equal(t, wantedManifest, actualManifest)
 			},
@@ -375,6 +392,7 @@ subscribe:
 							},
 						},
 					},
+					Environments: map[string]*WorkerServiceConfig{},
 				}
 				require.Equal(t, wantedManifest, actualManifest)
 			},
@@ -402,6 +420,64 @@ type: 'OH NO'
 	}
 }
 
+func TestStringOrFromCFN_UnmarshalYAML(t *testing.T) {
+	type mockField struct {
+		stringOrFromCFN
+	}
+	type mockParentField struct {
+		MockField mockField `yaml:"mock_field"`
+	}
+	testCases := map[string]struct {
+		in          []byte
+		wanted      mockParentField
+		wantedError error
+	}{
+		"unmarshal plain string": {
+			in: []byte(`mock_field: hey`),
+			wanted: mockParentField{
+				MockField: mockField{
+					stringOrFromCFN{
+						Plain: aws.String("hey"),
+					},
+				},
+			},
+		},
+		"unmarshal import name": {
+			in: []byte(`mock_field:
+  from_cfn: yo`),
+			wanted: mockParentField{
+				MockField: mockField{
+					stringOrFromCFN{
+						FromCFN: fromCFN{
+							Name: aws.String("yo"),
+						},
+					},
+				},
+			},
+		},
+		"nothing to unmarshal": {
+			in: []byte(`other_field: yo`),
+		},
+		"fail to unmarshal": {
+			in: []byte(`mock_field:
+  heyyo: !`),
+			wantedError: errors.New(`cannot unmarshal field to a string or into a map`),
+		},
+	}
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			var s mockParentField
+			err := yaml.Unmarshal(tc.in, &s)
+			if tc.wantedError != nil {
+				require.EqualError(t, err, tc.wantedError.Error())
+			} else {
+				require.NoError(t, err)
+				require.Equal(t, tc.wanted, s)
+			}
+		})
+	}
+}
+
 func TestCount_UnmarshalYAML(t *testing.T) {
 	var (
 		perc               = Percentage(70)
