@@ -97,6 +97,7 @@ var (
 		"http-listener",
 		"env-controller",
 		"mount-points",
+		"variables",
 		"volumes",
 		"image-overrides",
 		"instancerole",
@@ -106,6 +107,7 @@ var (
 		"nlb",
 		"vpc-connector",
 		"alb",
+		"rollback-alarms",
 	}
 
 	// Operating systems to determine Fargate platform versions.
@@ -132,7 +134,7 @@ type SidecarOpts struct {
 	Port         *string
 	Protocol     *string
 	CredsParam   *string
-	Variables    map[string]string
+	Variables    map[string]Variable
 	Secrets      map[string]Secret
 	Storage      SidecarStorageOpts
 	DockerLabels map[string]string
@@ -210,7 +212,7 @@ type LogConfigOpts struct {
 	EnableMetadata *string
 	SecretOptions  map[string]Secret
 	ConfigFile     *string
-	Variables      map[string]string
+	Variables      map[string]Variable
 	Secrets        map[string]Secret
 }
 
@@ -252,30 +254,111 @@ func (h HTTPHealthCheckOpts) IsHTTPS() bool {
 	return h.Port == "443"
 }
 
-// A Secret represents an SSM or SecretsManager secret that can be rendered in CloudFormation.
-type Secret interface {
+type importable interface {
+	RequiresImport() bool
+}
+
+type importableValue interface {
+	importable
+	Value() string
+}
+
+// Variable represents the value of an environment variable.
+type Variable importableValue
+
+// ImportedVariable returns a Variable that should be imported from a stack.
+func ImportedVariable(name string) Variable {
+	return importedEnvVar(name)
+}
+
+// PlainVariable returns a Variable that is a plain string value.
+func PlainVariable(value string) Variable {
+	return plainEnvVar(value)
+}
+
+type plainEnvVar string
+
+// RequiresImport returns false for a plain string environment variable.
+func (v plainEnvVar) RequiresImport() bool {
+	return false
+}
+
+// Value returns the plain string value of the environment variable.
+func (v plainEnvVar) Value() string {
+	return string(v)
+}
+
+type importedEnvVar string
+
+// RequiresImport returns true for an imported environment variable.
+func (v importedEnvVar) RequiresImport() bool {
+	return true
+}
+
+// Value returns the name of the import that will be the value of the environment variable.
+func (v importedEnvVar) Value() string {
+	return string(v)
+}
+
+type importableSubValueFrom interface {
+	importable
 	RequiresSub() bool
 	ValueFrom() string
 }
 
-// ssmOrSecretARN is a Secret stored that can be referred by an SSM Parameter Name or a secret ARN.
-type ssmOrSecretARN struct {
+// A Secret represents an SSM or SecretsManager secret that can be rendered in CloudFormation.
+type Secret importableSubValueFrom
+
+// plainSSMOrSecretARN is a Secret stored that can be referred by an SSM Parameter Name or a secret ARN.
+type plainSSMOrSecretARN struct {
 	value string
 }
 
 // RequiresSub returns true if the secret should be populated in CloudFormation with !Sub.
-func (s ssmOrSecretARN) RequiresSub() bool {
+func (s plainSSMOrSecretARN) RequiresSub() bool {
 	return false
 }
 
-// ValueFrom returns the valueFrom field for the secret.
-func (s ssmOrSecretARN) ValueFrom() string {
+// RequiresImport returns true if the secret should be imported from other CloudFormation stack.
+func (s plainSSMOrSecretARN) RequiresImport() bool {
+	return false
+}
+
+// ValueFrom returns the plain string value of the secret.
+func (s plainSSMOrSecretARN) ValueFrom() string {
 	return s.value
 }
 
-// SecretFromSSMOrARN returns a Secret that refers to an SSM parameter or a secret ARN.
-func SecretFromSSMOrARN(value string) ssmOrSecretARN {
-	return ssmOrSecretARN{
+// SecretFromPlainSSMOrARN returns a Secret that refers to an SSM parameter or a secret ARN.
+func SecretFromPlainSSMOrARN(value string) plainSSMOrSecretARN {
+	return plainSSMOrSecretARN{
+		value: value,
+	}
+}
+
+// importedSSMorSecretARN is a Secret that can be referred by the name of the import value from env addon or an arbitary CloudFormation stack.
+type importedSSMorSecretARN struct {
+	value string
+}
+
+// RequiresSub returns true if the secret should be populated in CloudFormation with !Sub.
+func (s importedSSMorSecretARN) RequiresSub() bool {
+	return false
+}
+
+// RequiresImport returns true if the secret should be imported from env addon or an arbitary CloudFormation stack.
+func (s importedSSMorSecretARN) RequiresImport() bool {
+	return true
+}
+
+// ValueFrom returns the name of the import value of the Secret.
+func (s importedSSMorSecretARN) ValueFrom() string {
+	return s.value
+}
+
+// SecretFromImportedSSMOrARN returns a Secret that refers to imported name of SSM parameter or a secret ARN.
+func SecretFromImportedSSMOrARN(value string) importedSSMorSecretARN {
+	return importedSSMorSecretARN{
 		value: value,
 	}
 }
@@ -290,6 +373,11 @@ func (s secretsManagerName) RequiresSub() bool {
 	return true
 }
 
+// RequiresImport returns true if the secret should be imported from other CloudFormation stack.
+func (s secretsManagerName) RequiresImport() bool {
+	return false
+}
+
 // ValueFrom returns the resource ID of the SecretsManager secret for populating the ARN.
 func (s secretsManagerName) ValueFrom() string {
 	return fmt.Sprintf("secret:%s", s.value)
@@ -405,12 +493,41 @@ type ObservabilityOpts struct {
 	Tracing string // The name of the vendor used for tracing.
 }
 
-// DeploymentConfiguraitonOpts holds values for MinHealthyPercent and MaxPercent.
+// DeploymentConfigurationOpts holds configuration for rolling deployments.
 type DeploymentConfigurationOpts struct {
 	// The lower limit on the number of tasks that should be running during a service deployment or when a container instance is draining.
 	MinHealthyPercent int
 	// The upper limit on the number of tasks that should be running during a service deployment or when a container instance is draining.
 	MaxPercent int
+	Rollback   RollingUpdateRollbackConfig
+}
+
+// RollingUpdateRollbackConfig holds config for rollback alarms.
+type RollingUpdateRollbackConfig struct {
+	AlarmNames []string // Names of existing alarms.
+
+	// Custom alarms to create.
+	CPUUtilization    *float64
+	MemoryUtilization *float64
+}
+
+// HasRollbackAlarms returns true if the client is using ABR.
+func (cfg RollingUpdateRollbackConfig) HasRollbackAlarms() bool {
+	return len(cfg.AlarmNames) > 0 || cfg.HasCustomAlarms()
+}
+
+// HasCustomAlarms returns true if the client is using Copilot-generated alarms for alarm-based rollbacks.
+func (cfg RollingUpdateRollbackConfig) HasCustomAlarms() bool {
+	return cfg.CPUUtilization != nil || cfg.MemoryUtilization != nil
+}
+
+// TruncateAlarmName ensures that alarm names don't exceed the 255 character limit.
+func (cfg RollingUpdateRollbackConfig) TruncateAlarmName(app, env, svc, alarmType string) string {
+	if len(app)+len(env)+len(svc)+len(alarmType) <= 255 {
+		return fmt.Sprintf("%s-%s-%s-%s", app, env, svc, alarmType)
+	}
+	maxSubstringLength := (255 - len(alarmType) - 3) / 3
+	return fmt.Sprintf("%s-%s-%s-%s", app[:maxSubstringLength], env[:maxSubstringLength], svc[:maxSubstringLength], alarmType)
 }
 
 // ExecuteCommandOpts holds configuration that's needed for ECS Execute Command.
@@ -440,7 +557,7 @@ type Topic struct {
 	Svc       string
 }
 
-// Fifo holds configuration needed if the topic is FIFO.
+// FIFOTopicConfig holds configuration needed if the topic is FIFO.
 type FIFOTopicConfig struct {
 	ContentBasedDeduplication *bool
 }
@@ -478,7 +595,7 @@ type SQSQueue struct {
 	FIFOQueueConfig *FIFOQueueConfig
 }
 
-// FifoAdvanceConfigOrBool holds information needed to render a FIFO SQS Queue in a container definition.
+// FIFOQueueConfig holds information needed to render a FIFO SQS Queue in a container definition.
 type FIFOQueueConfig struct {
 	FIFOThroughputLimit       *string
 	ContentBasedDeduplication *bool
@@ -492,7 +609,7 @@ type DeadLetterQueue struct {
 
 // NetworkOpts holds AWS networking configuration for the workloads.
 type NetworkOpts struct {
-	SecurityGroups []string
+	SecurityGroups []SecurityGroup
 	AssignPublicIP string
 	// SubnetsType and SubnetIDs are mutually exclusive. They won't be set together.
 	SubnetsType              string
@@ -500,6 +617,43 @@ type NetworkOpts struct {
 	DenyDefaultSecurityGroup bool
 }
 
+// SecurityGroup represents the ID of an additional security group associated with the tasks.
+type SecurityGroup importableValue
+
+// PlainSecurityGroup returns a SecurityGroup that is a plain string value.
+func PlainSecurityGroup(value string) SecurityGroup {
+	return plainSecurityGroup(value)
+}
+
+// ImportedSecurityGroup returns a SecurityGroup that should be imported from a stack.
+func ImportedSecurityGroup(name string) SecurityGroup {
+	return importedSecurityGroup(name)
+}
+
+type plainSecurityGroup string
+
+// RequiresImport returns false for a plain string SecurityGroup.
+func (sg plainSecurityGroup) RequiresImport() bool {
+	return false
+}
+
+// Value returns the plain string value of the SecurityGroup.
+func (sg plainSecurityGroup) Value() string {
+	return string(sg)
+}
+
+type importedSecurityGroup string
+
+// RequiresImport returns true for an imported SecurityGroup.
+func (sg importedSecurityGroup) RequiresImport() bool {
+	return true
+}
+
+// Value returns the name of the import that will be the value of the SecurityGroup.
+func (sg importedSecurityGroup) Value() string {
+	return string(sg)
+}
+
 // RuntimePlatformOpts holds configuration needed for Platform configuration.
 type RuntimePlatformOpts struct {
 	OS   string
@@ -546,7 +700,7 @@ type WorkloadOpts struct {
 	EnvVersion         string
 
 	// Additional options that are common between **all** workload templates.
-	Variables                map[string]string
+	Variables                map[string]Variable
 	Secrets                  map[string]Secret
 	Aliases                  []string
 	HTTPSListener            bool
