@@ -7,6 +7,8 @@ import (
 	"fmt"
 	"testing"
 
+	"github.com/spf13/afero"
+
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/stretchr/testify/require"
 )
@@ -16,51 +18,56 @@ func TestTemplate_ParseSvc(t *testing.T) {
 		testSvcName = "backend"
 	)
 	testCases := map[string]struct {
-		fs            func() map[string][]byte
+		fs            func() afero.Fs
 		wantedContent string
 		wantedErr     error
 	}{
 		"renders all common templates": {
-			fs: func() map[string][]byte {
+			fs: func() afero.Fs {
 				var baseContent string
 				for _, name := range partialsWorkloadCFTemplateNames {
 					baseContent += fmt.Sprintf(`{{include "%s" . | indent 2}}`+"\n", name)
 				}
 
-				return map[string][]byte{
-					"templates/workloads/services/backend/cf.yml":                         []byte(baseContent),
-					"templates/workloads/partials/cf/loggroup.yml":                        []byte("loggroup"),
-					"templates/workloads/partials/cf/envvars-container.yml":               []byte("envvars-container"),
-					"templates/workloads/partials/cf/envvars-common.yml":                  []byte("envvars-common"),
-					"templates/workloads/partials/cf/secrets.yml":                         []byte("secrets"),
-					"templates/workloads/partials/cf/executionrole.yml":                   []byte("executionrole"),
-					"templates/workloads/partials/cf/taskrole.yml":                        []byte("taskrole"),
-					"templates/workloads/partials/cf/workload-container.yml":              []byte("workload-container"),
-					"templates/workloads/partials/cf/fargate-taskdef-base-properties.yml": []byte("fargate-taskdef-base-properties"),
-					"templates/workloads/partials/cf/service-base-properties.yml":         []byte("service-base-properties"),
-					"templates/workloads/partials/cf/servicediscovery.yml":                []byte("servicediscovery"),
-					"templates/workloads/partials/cf/addons.yml":                          []byte("addons"),
-					"templates/workloads/partials/cf/sidecars.yml":                        []byte("sidecars"),
-					"templates/workloads/partials/cf/logconfig.yml":                       []byte("logconfig"),
-					"templates/workloads/partials/cf/autoscaling.yml":                     []byte("autoscaling"),
-					"templates/workloads/partials/cf/state-machine-definition.json.yml":   []byte("state-machine-definition"),
-					"templates/workloads/partials/cf/eventrule.yml":                       []byte("eventrule"),
-					"templates/workloads/partials/cf/state-machine.yml":                   []byte("state-machine"),
-					"templates/workloads/partials/cf/efs-access-point.yml":                []byte("efs-access-point"),
-					"templates/workloads/partials/cf/https-listener.yml":                  []byte("https-listener"),
-					"templates/workloads/partials/cf/http-listener.yml":                   []byte("http-listener"),
-					"templates/workloads/partials/cf/env-controller.yml":                  []byte("env-controller"),
-					"templates/workloads/partials/cf/mount-points.yml":                    []byte("mount-points"),
-					"templates/workloads/partials/cf/volumes.yml":                         []byte("volumes"),
-					"templates/workloads/partials/cf/image-overrides.yml":                 []byte("image-overrides"),
-					"templates/workloads/partials/cf/instancerole.yml":                    []byte("instancerole"),
-					"templates/workloads/partials/cf/accessrole.yml":                      []byte("accessrole"),
-					"templates/workloads/partials/cf/publish.yml":                         []byte("publish"),
-					"templates/workloads/partials/cf/subscribe.yml":                       []byte("subscribe"),
-					"templates/workloads/partials/cf/nlb.yml":                             []byte("nlb"),
-					"templates/workloads/partials/cf/vpc-connector.yml":                   []byte("vpc-connector"),
-					"templates/workloads/partials/cf/alb.yml":                             []byte("alb"),
-				}
+				fs := afero.NewMemMapFs()
+				_ = fs.MkdirAll("templates/workloads/services/backend/", 0755)
+				_ = fs.MkdirAll("templates/workloads/partials/cf/", 0755)
+				_ = afero.WriteFile(fs, "templates/workloads/services/backend/cf.yml", []byte(baseContent), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/loggroup.yml", []byte("loggroup"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/envvars-container.yml", []byte("envvars-container"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/envvars-common.yml", []byte("envvars-common"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/secrets.yml", []byte("secrets"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/executionrole.yml", []byte("executionrole"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/taskrole.yml", []byte("taskrole"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/workload-container.yml", []byte("workload-container"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/fargate-taskdef-base-properties.yml", []byte("fargate-taskdef-base-properties"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/service-base-properties.yml", []byte("service-base-properties"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/servicediscovery.yml", []byte("servicediscovery"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/addons.yml", []byte("addons"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/sidecars.yml", []byte("sidecars"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/logconfig.yml", []byte("logconfig"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/autoscaling.yml", []byte("autoscaling"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/state-machine-definition.json.yml", []byte("state-machine-definition"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/eventrule.yml", []byte("eventrule"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/state-machine.yml", []byte("state-machine"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/efs-access-point.yml", []byte("efs-access-point"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/https-listener.yml", []byte("https-listener"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/http-listener.yml", []byte("http-listener"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/env-controller.yml", []byte("env-controller"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/mount-points.yml", []byte("mount-points"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/variables.yml", []byte("variables"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/volumes.yml", []byte("volumes"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/image-overrides.yml", []byte("image-overrides"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/instancerole.yml", []byte("instancerole"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/accessrole.yml", []byte("accessrole"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/publish.yml", []byte("publish"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/subscribe.yml", []byte("subscribe"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/nlb.yml", []byte("nlb"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/vpc-connector.yml", []byte("vpc-connector"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/alb.yml", []byte("alb"), 0644)
+				_ = afero.WriteFile(fs, "templates/workloads/partials/cf/rollback-alarms.yml", []byte("rollback-alarms"), 0644)
+
+				return fs
 			},
 			wantedContent: `  loggroup
   envvars-container
@@ -84,6 +91,7 @@ func TestTemplate_ParseSvc(t *testing.T) {
   http-listener
   env-controller
   mount-points
+  variables
   volumes
   image-overrides
   instancerole
@@ -93,6 +101,7 @@ func TestTemplate_ParseSvc(t *testing.T) {
   nlb
   vpc-connector
   alb
+  rollback-alarms
 `,
 		},
 	}
@@ -101,7 +110,7 @@ func TestTemplate_ParseSvc(t *testing.T) {
 		t.Run(name, func(t *testing.T) {
 			// GIVEN
 			tpl := &Template{
-				fs: &mockReadFileFS{tc.fs()},
+				fs: &mockFS{tc.fs()},
 			}
 
 			// WHEN
@@ -135,7 +144,7 @@ func TestHasSecrets(t *testing.T) {
 		"service has secrets": {
 			in: WorkloadOpts{
 				Secrets: map[string]Secret{
-					"hello": SecretFromSSMOrARN("world"),
+					"hello": SecretFromPlainSSMOrARN("world"),
 				},
 			},
 			wanted: true,
@@ -263,18 +272,38 @@ func TestHTTPTargetContainer_IsHTTPS(t *testing.T) {
 	require.False(t, HTTPTargetContainer{Port: "8080"}.IsHTTPS())
 }
 
-func TestSsmOrSecretARN_RequiresSub(t *testing.T) {
-	require.False(t, ssmOrSecretARN{}.RequiresSub(), "SSM Parameter Store or secret ARNs do not require !Sub")
+func TestPlainSSMOrSecretARN_RequiresSub(t *testing.T) {
+	require.False(t, plainSSMOrSecretARN{}.RequiresSub(), "plain SSM Parameter Store or secret ARNs do not require !Sub")
 }
 
-func TestSsmOrSecretARN_ValueFrom(t *testing.T) {
-	require.Equal(t, "/github/token", SecretFromSSMOrARN("/github/token").ValueFrom())
+func TestPlainSSMOrSecretARN_RequiresImport(t *testing.T) {
+	require.False(t, plainSSMOrSecretARN{}.RequiresImport(), "plain SSM Parameter Store or secret ARNs do not require !ImportValue")
+}
+
+func TestPlainSSMOrSecretARN_ValueFrom(t *testing.T) {
+	require.Equal(t, "/github/token", SecretFromPlainSSMOrARN("/github/token").ValueFrom())
+}
+
+func TestImportedSSMOrSecretARN_RequiresSub(t *testing.T) {
+	require.False(t, importedSSMorSecretARN{}.RequiresSub(), "imported SSM Parameter Store or secret ARNs do not require !Sub")
+}
+
+func TestImportedSSMOrSecretARN_RequiresImport(t *testing.T) {
+	require.True(t, importedSSMorSecretARN{}.RequiresImport(), "imported SSM Parameter Store or secret ARNs requires !ImportValue")
+}
+
+func TestImportedSSMOrSecretARN_ValueFrom(t *testing.T) {
+	require.Equal(t, "stack-SSMGHTokenName", SecretFromImportedSSMOrARN("stack-SSMGHTokenName").ValueFrom())
 }
 
 func TestSecretsManagerName_RequiresSub(t *testing.T) {
 	require.True(t, secretsManagerName{}.RequiresSub(), "secrets referring to a SecretsManager name need to be expanded to a full ARN")
 }
 
+func TestSecretsManagerName_RequiresImport(t *testing.T) {
+	require.False(t, secretsManagerName{}.RequiresImport(), "secrets referring to a SecretsManager name do not require !ImportValue")
+}
+
 func TestSecretsManagerName_Service(t *testing.T) {
 	require.Equal(t, "secretsmanager", secretsManagerName{}.Service())
 }
@@ -419,3 +448,35 @@ func TestEnvControllerParameters(t *testing.T) {
 		})
 	}
 }
+
+func TestRollingUpdateRollbackConfig_TruncateAlarmName(t *testing.T) {
+	testCases := map[string]struct {
+		config   RollingUpdateRollbackConfig
+		inApp    string
+		inEnv    string
+		inSvc    string
+		inAlarmType string
+		expected string
+	}{
+		"with no need to truncate": {
+			inApp: "shortAppName",
+			inEnv: "shortEnvName",
+			inSvc: "shortSvcName",
+			inAlarmType: "CopilotRollbackMemAlarm",
+			expected: "shortAppName-shortEnvName-shortSvcName-CopilotRollbackMemAlarm",
+		},
+		"with need to truncate at 76 chars per element": {
+			inApp: "12345678911234567892123456789312345678941234567895123456789612345678971234567898",
+			inEnv: "12345678911234567892123456789312345678941234567895123456789612345678971234567898",
+			inSvc: "12345678911234567892123456789312345678941234567895123456789612345678971234567898",
+			inAlarmType: "CopilotRollbackCPUAlarm",
+			expected: "1234567891123456789212345678931234567894123456789512345678961234567897123456-1234567891123456789212345678931234567894123456789512345678961234567897123456-1234567891123456789212345678931234567894123456789512345678961234567897123456-CopilotRollbackCPUAlarm",
+		},
+		
+	}
+	for name, tc := range testCases {
+	t.Run(name, func(t *testing.T) {
+		require.Equal(t, tc.expected, tc.config.TruncateAlarmName(tc.inApp, tc.inEnv, tc.inSvc, tc.inAlarmType))
+	})}
+}
+
